/*---------------------------------------------------------------------------

	File:
		astrob.cpp
	
	Abstract:
		Implements the "AstroBlaster" game driver functions.

	Revisions:
		01-Apr-97 EAM Created

	Copyright (C) 1997 Edward Massey

---------------------------------------------------------------------------*/

#include "pch.h"
#include <stdio.h>
#include "misc.h"
#include "z80api.h"
#include "game.h"
#include "game1.h"

static BYTE _rgbChars[256][8][8];
static BYTE _rgbSprites[512][16][16];
static BYTE _rgbSpriteMask[512][16][16];

#include "blit.h"

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static const BYTE _rgbColorPROM[] =
{
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
	0x00, 0xFF, 0x07, 0x38,
};

static void BuildCompositePalette(PALETTEENTRY *ppeOut, UINT &cpeOut)
{
	for (cpeOut = 0; cpeOut < DIM(_rgbColorPROM); cpeOut++)
	{
		int i = cpeOut;

		int bit0,bit1,bit2;

		bit0 = (_rgbColorPROM[i] >> 0) & 0x01;
		bit1 = (_rgbColorPROM[i] >> 1) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 2) & 0x01;

		ppeOut[cpeOut].peRed = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;

		bit0 = (_rgbColorPROM[i] >> 3) & 0x01;
		bit1 = (_rgbColorPROM[i] >> 4) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 5) & 0x01;

		ppeOut[cpeOut].peGreen = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;

		bit0 = 0;
		bit1 = (_rgbColorPROM[i] >> 6) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 7) & 0x01;

		ppeOut[cpeOut].peBlue  = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	}
}

static HRESULT Init(GAME_LOAD_INFO &gli)
{
	static const TCHAR szPath[] = "astrob";

	static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
//	static const RECT _rcDisplay = { 0, 0, 256 + 16, 256 + 16};
	static const RECT _rcDisplay = { 32, 16, 256 - 32, 256 + 16};

	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	BuildCompositePalette(gli.ppe, gli.cpe);

	//
	//
	//
	
/*
	838 - looks like code.  has a charset in it though
	842 - looks like code.  has a charset in it though

	845 - looks like graphics
	846 - looks like graphics
	847 - looks like graphics
	849 - looks like graphics
	850 - looks like graphics

*/

	RAGE_ROMTABLE_BEGIN(ROMs)
		RAGE_ROMTABLE_ENTRY("837", 0x0000, 0x0800)
		RAGE_ROMTABLE_ENTRY("838", 0x0800, 0x0800)
		RAGE_ROMTABLE_ENTRY("839", 0x1000, 0x0800)
		RAGE_ROMTABLE_ENTRY("840", 0x1800, 0x0800)
		RAGE_ROMTABLE_ENTRY("841", 0x2000, 0x0800)
		RAGE_ROMTABLE_ENTRY("842", 0x2800, 0x0800)
		RAGE_ROMTABLE_ENTRY("843", 0x3000, 0x0800)
		RAGE_ROMTABLE_ENTRY("844", 0x3800, 0x0800)
	RAGE_ROMTABLE_END

	if (readroms(RAM, ROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	//
	//
	//
	
	BYTE tmpstorage[0x6000];

	RAGE_ROMTABLE_BEGIN(GFXROMs)
		RAGE_ROMTABLE_ENTRY("852", 0x0000, 0x0800)
		RAGE_ROMTABLE_ENTRY("851", 0x0000, 0x0800)
		RAGE_ROMTABLE_ENTRY("850", 0x0000, 0x0800)
	RAGE_ROMTABLE_END

	if (readroms(tmpstorage, GFXROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	static const MY_GFX_DECODE mgdChars =
	{
		{ 8, 8 },
		256,
		8 * 8,
		2,
		8 * 8 * 256,
		2,
		{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
		{ 0, 1, 2, 3, 4, 5, 6, 7 },
	};

	MyGfxDecode(tmpstorage, mgdChars, (LPBYTE)_rgbChars);

/*
	static const MY_GFX_DECODE mgdSprites =
	{
		{ 16, 16 },
		128,
		64 * 8,
		2,
		4,
		2,
		{ 0*16, 2*16, 4*16, 6*16, 8*16, 10*16, 12*16, 14*16,
			16*16, 18*16, 20*16, 22*16, 24*16, 26*16, 28*16, 30*16 },
		{ 24+0, 24+1, 24+2, 24+3, 16+0, 16+1, 16+2, 16+3,
			8+0, 8+1, 8+2, 8+3, 0, 1, 2, 3 },
	};

	static const int rgiMirror[] = {
		0 * mgdSprites.cImage,
		1 * mgdSprites.cImage,
		2 * mgdSprites.cImage,
		3 * mgdSprites.cImage
	};

	static const int cImageInc = 1;

	MyGfxDecode2(tmpstorage + 0x4000, mgdSprites, cImageInc, rgiMirror,
		(LPBYTE)_rgbSprites, (LPBYTE)_rgbSpriteMask);
*/

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static BYTE M_RDMEM(WORD A)
{
	switch (A)
	{
/*
		case 0xA002:
		{
			BYTE b = 0xDF;

			if (osd_key_pressed(OSD_KEY_F1))
				b ^= 0x04;

			return b;
		}
*/
		 
		default:
			return RAM[A];
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static void M_WRMEM(WORD A, BYTE V)
{
	if (A < 0x4000)
		return;

	RAM[A] = V;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static int interrupt_vector = 0xff;

static int Interrupt(void)
{
	if (!CheckMessages())
		CPURunning = 0;

	Throttle(60); // FRAMES_PER_SECOND);

	return interrupt_vector;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

// TODO:  call higher-level Exec function that handles the common parts
//        of Interrupt, pass it our CPU speed and frame rate

static HRESULT Run(LPVOID pvContext)
{
	EXEC_Z80_STRUCT es;
	
	ZERO_INIT(es);
	
	es.nCallbackFreq = 66000; // CALLBACK_FREQ;
	es.pfnInterrupt = Interrupt;
	es.pfnReadMemory = M_RDMEM;
	es.pfnWriteMemory = M_WRMEM;
	es.rgbMemory = RAM;
	
	return ExecZ80(es);
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static HRESULT RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
	int i, j;

	if (GetAsyncKeyState('G') < 0)
	{
		if (GetAsyncKeyState('1') < 0)
		{
			for (i = 0; i < 512; i++)
				DrawChar(rgbBuffer, lPitch, 8 * (i % 32), 8 * (i / 32), i, 0);
		}

		if (GetAsyncKeyState('2') < 0)
		{
			for (i = 0; i < 128; i++)
				DrawSprite(rgbBuffer, lPitch, 16 * (i % 16), 16 * (i / 16), i, 0);
		}

		return S_OK;
	}

	for (i = 0; i < 32 * 32; i++)
	{
		DrawChar((LPBYTE)_rgbChars, RAM[0x8000 + i], 0,
			8 * (i / 32), 8 * (31 - (i % 32)), rgbBuffer, lPitch);
	}

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

GAME_DESCRIPTOR _gmdAstroBlaster = 
{
	TEXT("AstroBlaster"),
	Init,
	Run,
	RenderScene,
	NULL,
	NULL,
	NULL,
	NULL
};

