/*---------------------------------------------------------------------------

	File:
		galaxian.cpp
	
	Abstract:
		Implements the Galaxian game driver functions.

	Revisions:
		31-Jan-97 EAM Created
		09-Feb-97 EAM Optimizations... 120 fps (unthrottled, DirectDraw)
		26-Apr-97 EAM Migrated to MageX v0.5 model

	Copyright (C) 1997 Edward Massey

	TODO:  better sound
	TODO:  left shift the sprite data instead of the color scheme selector
	       to reduce the amount of math
	TODO:  draw sprites on TOP of bullets!!!

---------------------------------------------------------------------------*/

#include "pch.h"
#include "misc.h"
#include "galaxian.h"
#include "game.h"
#include "game1.h"
#include "../z80/execz80.h"
#include "stars.h"

static IMAGE *_pimgSprites;
static IMAGE *_pimgChars;

/* default dip switches settings */
static BYTE _rgbDSW[] = { 0x00 };

static STARS _stars;

/*---------------------------------------------------------------------------

	Static data

---------------------------------------------------------------------------*/

static const BYTE _rgbColorPROM[] =
{
	/* palette */
	0x00,0x00,0x00,0xF6,
	0x00,0x16,0xC0,0x3F,
	0x00,0xD8,0x07,0x3F,
	0x00,0xC0,0xC4,0x07,
	0x00,0xC0,0xA0,0x0C,
	0x00,0x00,0x00,0x07,
	0x00,0xF6,0x07,0xF0,
	0x00,0x76,0x07,0xC6
};

static void BuildCompositePalette(PALETTEENTRY *ppeOut, UINT &cpeOut)
{
	for (cpeOut = 0; cpeOut < DIM(_rgbColorPROM); cpeOut++)
	{
		int i = cpeOut;

		int bit0,bit1,bit2;

		bit0 = (_rgbColorPROM[i] >> 0) & 0x01;
		bit1 = (_rgbColorPROM[i] >> 1) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 2) & 0x01;

		ppeOut[cpeOut].peRed = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;

		bit0 = (_rgbColorPROM[i] >> 3) & 0x01;
		bit1 = (_rgbColorPROM[i] >> 4) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 5) & 0x01;

		ppeOut[cpeOut].peGreen = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;

		bit0 = 0;
		bit1 = (_rgbColorPROM[i] >> 6) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 7) & 0x01;

		ppeOut[cpeOut].peBlue  = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	}
}

enum 
{
    black, red, purple, cyan, yellow, blue, green, white
};

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

//
//
//

static const TCHAR _szPath[] = "galaxian";

RAGE_ROMTABLE_BEGIN(ROMs)
	RAGE_ROMTABLE_ENTRY("galaxian.u", 0x0000, 0x0800)
	RAGE_ROMTABLE_ENTRY("galaxian.v", 0x0800, 0x0800)
	RAGE_ROMTABLE_ENTRY("galaxian.w", 0x1000, 0x0800)
	RAGE_ROMTABLE_ENTRY("galaxian.y", 0x1800, 0x0800)
	RAGE_ROMTABLE_ENTRY("galaxian.z", 0x2000, 0x0800)
RAGE_ROMTABLE_END

RAGE_ROMTABLE_BEGIN(GFXROMs)
	RAGE_ROMTABLE_ENTRY("galaxian.1j", 0x0000, 0x0800)
	RAGE_ROMTABLE_ENTRY("galaxian.1k", 0x0800, 0x0800)
RAGE_ROMTABLE_END

static IExecZ80 *_pZ80;

static BYTE M_RDMEM(WORD A);
static void M_WRMEM(WORD A, BYTE);

static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
static const RECT _rcDisplay = { 16, 0, 256 - 16, 256 };

static const DECODE_GRAPHICS dgsSprites =
{
	0x00000001,
	{
		{ 16, 16 },
		64,
		32 * 8,
		2,
		{ 0, 64 * 16 * 16 },
		2,
		{ 23*8, 22*8, 21*8, 20*8, 19*8, 18*8, 17*8, 16*8,
				7*8, 6*8, 5*8, 4*8, 3*8, 2*8, 1*8, 0*8 },
		{ 0, 1, 2, 3, 4, 5, 6, 7,
				8*8+0, 8*8+1, 8*8+2, 8*8+3, 8*8+4, 8*8+5, 8*8+6, 8*8+7 },
	},

	1,
	
	4, 
	
	{
		0, 64, 128, 3 * 64
	},
	0, 
	NULL
};

static const DECODE_GRAPHICS dgsChars =
{
	0x00000001,
	{
		{ 8, 8 },
		256,
		8 * 8,
		2,
		{ 0, 8 * 8 * 256 },
		2,
		{ 7*8, 6*8, 5*8, 4*8, 3*8, 2*8, 1*8, 0*8 },
		{ 0, 1, 2, 3, 4, 5, 6, 7 },
	},

	1,
	
	1, 
	
	{
		0
	},
	0, 
	NULL
};

static const EXECZ80_MEM_DESCRIPTOR rgmd[] = 
{
#if 0
	{ 0x0000, 0xFFFF, 0, M_RDMEM, M_WRMEM },
#else
	{ 0x0000, 0xFFFF, 0, NULL, NULL },
	{ 0x0000, 0x2800, 0, NULL, M_WRMEM },
	{ 0x6000, 0x6000, 0, M_RDMEM, NULL },
	{ 0x6800, 0x6800, 0, M_RDMEM, NULL },
	{ 0x7000, 0x7000, 0, M_RDMEM, NULL },
	{ 0, 0, 0, (EXECZ80_MEMREAD_PROC)0xFFFFFFFF, 0 },
#endif
};

static const BYTE _rgbEnvelope[32] =
{
	0x00,0x00,0x00,0x00,0x22,0x22,0x22,0x22,0x44,0x44,0x44,0x44,0x22,0x22,0x22,0x22,
	0x00,0x00,0x00,0x00,0xdd,0xdd,0xdd,0xdd,0xbb,0xbb,0xbb,0xbb,0xdd,0xdd,0xdd,0xdd
};

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static HRESULT CommonInit(LPCTSTR pszPath, const RomModule *pCodeRoms,
	const RomModule *pGfxRoms, GAME_LOAD_INFO &gli)
{
	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	BuildCompositePalette(gli.ppe, gli.cpe);

	if (readroms(RAM, pCodeRoms, pszPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	BYTE tmpstorage[0x4000];

	if (readroms(tmpstorage, pGfxRoms, pszPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	//
	//
	//

	MyGfxDecode3(dgsChars, tmpstorage, _pimgChars);
	MyGfxDecode3(dgsSprites, tmpstorage, _pimgSprites);

	//
	//
	//
	
	EXECZ80_CREATE cs;

	ZERO_INIT(cs);

	cs.rgbMemory = RAM;
	cs.pfnMemRead = M_RDMEM;
	cs.pfnMemWrite = M_WRMEM;
	cs.pmd = rgmd;
	
	HRESULT hr = EclCreateZ80(cs, _pZ80);

	if (FAILED(hr))
		return hr;

	//
	//
	//
	
	srand(timeGetTime());

	_stars.Init();
	
	_stars.m_nScrollRate = 2;

	// NOTE:  find matching color in lookup table...
	
	for (int i = 0; i < _stars.m_cStars; i++)
	{
		while (_rgbColorPROM[_stars.m_rgStars[i].c] == 0)
			_stars.m_rgStars[i].c = rand() % 32;
	}

	return S_OK;
}

HRESULT GalaxianInit(GAME_LOAD_INFO &gli)
{
	return CommonInit(_szPath, ROMs, GFXROMs, gli);
}

void GalaxianTerm()
{
	_pZ80->Release();

	FreeImage(_pimgChars);
	FreeImage(_pimgSprites);
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static const CONTROL _rgControl[] =
{
	{ GALAXIAN_IN0_PORT, 0x00, OSD_KEY_3, OSD_KEY_3, OSD_KEY_LEFT, OSD_KEY_RIGHT, OSD_KEY_CONTROL, 0, OSD_KEY_F2, OSD_KEY_S },
	{ 0xFFFF }
};

static BYTE M_RDMEM(WORD A)
{
/*
	BYTE b;
	
	if (ReadControl(A, _rgControl, DIM(_rgControl), b))
		return b;
*/

	switch (A)
	{
		case GALAXIAN_IN0_PORT: // Switch 0
		{
			BYTE b = 0x00;

			if (osd_key_pressed(OSD_KEY_3))
				b ^= GALAXIAN_IN0_COIN1;

			if (osd_key_pressed(OSD_KEY_LEFT))
				b ^= GALAXIAN_IN0_P1_LEFT;

			if (osd_key_pressed(OSD_KEY_RIGHT))
				b ^= GALAXIAN_IN0_P1_RIGHT;

			if (osd_key_pressed(OSD_KEY_CONTROL) || osd_key_pressed(OSD_KEY_ALT))
				b ^= GALAXIAN_IN0_P1_SHOOT;

			if (osd_key_pressed(OSD_KEY_F9))
				b ^= GALAXIAN_IN0_TEST;

			return b;
		}

		case GALAXIAN_IN1_PORT: // Switch 1
		{
			BYTE b = 0x00;

			if (_rgbDSW[0] & 1)
				b ^= GALAXIAN_IN1_SW1;

			if (_rgbDSW[0] & 2)
				b ^= GALAXIAN_IN1_SW2;

			if (osd_key_pressed(OSD_KEY_1))
				b ^= GALAXIAN_IN1_1P_START;

			if (osd_key_pressed(OSD_KEY_2))
				b ^= GALAXIAN_IN1_2P_START;

			return b;
		}

		case GALAXIAN_DSW_PORT: // Dip Switch
		{
			BYTE b = 0x04;
			
			b |= _rgbDSW[0] >> 2;

			return b;
		}

		default:
			return RAM[A];
	}
}

static void M_WRMEM(WORD A, BYTE V)
{
	// Ignore writes to ROM
	if (A < GALAXIAN_END_OF_ROM)
		return;

	RAM[A] = V;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

void GalaxianUpdateSound(LPBYTE rgbBuffer, LONG cbBuffer)
{
	ULONG lRate = 11025;
	ULONG cShift = 12;
	ULONG cSamp = DIM(_rgbEnvelope) << cShift;
	ULONG nMul = cSamp / lRate;

	int nFreq = RAM[GALAXIAN_SOUND_FREQUENCY] * 5;

	if (RAM[GALAXIAN_SOUND_FREQUENCY] == 0xFF)
		nFreq = 0;

	ULONG iInc = nFreq * nMul;

	int nNoise = RAM[0x6803];

	static unsigned long iPos;

	while (cbBuffer--)
	{
		int nAcc = 128;

		nAcc += _rgbEnvelope[iPos >> cShift];

		iPos += iInc;

		if (iPos > cSamp)
			iPos -= cSamp;

/*
		if (nNoise & 0x01)
			nAcc += ;
*/

		//
		//  clip...
		//
		
//		nAcc /= 16 * 3;
		
		if (nAcc < 0)
			nAcc = 0;
		else if (nAcc > 255)
			nAcc = 255;

		//
		//
		//

		*rgbBuffer++ = nAcc;
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

int GalaxianExec()
{
	_stars.Exec();

	int cCycles;
	
	cCycles = _pZ80->Exec(50000);
	
	if (RAM[GALAXIAN_NMI] != 0)
		cCycles += _pZ80->Nmi();

	return cCycles;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

void GalaxianRenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
	LPBYTE pBase = &RAM[base];

	int X, Y, i;

	//
	//
	//

	LPDWORD pDest = (LPDWORD)rgbBuffer;

	for (Y = 0; Y < 256; Y++)
	{
		memset(pDest, 0, 256);
		pDest += lPitch / 4;
	}

	_stars.Draw(rgbBuffer + lPitch * 16, lPitch);

	//
	//  Draw 32x32 characters
	//

	for (Y = 0; Y < 32; Y++)
	{
		//
		//  NOTE:  x origin and color of each line of text is set in hardware
		//

		const int nShift = pBase[0x0800 + Y * 2];
		const int color = pBase[0x0801 + Y * 2] & 0x07;
		const int sy = Y * 8;

		for (X = 0; X < 32; X++)
		{
			const int offset_value = pBase[(31 - X) * 32 + Y];

			if (offset_value == 16)
				continue;

			const int sx = ((X * 8 + nShift) % 256);
			
//			_pimgChars->DrawChar(rgbBuffer, lPitch, sx, sy, offset_value, color << 2);
			DrawCharT(_pimgChars->rgbData, _pimgChars->rgbMask,
				offset_value, color << 2, sx, sy, rgbBuffer, lPitch);
		}
	}

	// draw sound register
//	memset(rgbBuffer + lPitch * 255, 4, RAM[0x7800]);

	pBase += 0x0840;

	//
	//  Draw sprites
	//

	struct SPRITE
	{
		BYTE x, i, c, y;
	};

	const SPRITE *ps = (SPRITE*)&RAM[0x5840];

	for (i = 0; i < 8; i++)
	{
		X = pBase[0];

		if (X != 0)
		{
			int spritenum = pBase[1] & 0x3F;

			if (spritenum != 0)
			{
				Y = pBase[3];

				if (Y != 0)
				{
					// NOTE:  use of prerotated sprite images means we can pass pBase[1] unmodified
					_pimgSprites->DrawSprite(rgbBuffer, lPitch, X, Y, pBase[1]/*spritenum*/, pBase[2] << 2);
				}
			}
		}

		pBase += 4;
	}

	//
	//  Draw Bullets
	//

	for (i = 0; i < 8; i++)
	{
		X = pBase[1];

		const int color = (i < 6) ? 3 : 7;

		if (X != 0 && X != 255)
		{
			Y = 256 - pBase[3] - 4;

			if (Y >= 0)
			{
				LPBYTE pBits = &rgbBuffer[lPitch * Y + X];

				for (int j = 0; j < 3; j++)
				{
					*pBits = color;

					pBits += lPitch;
				}
			}
		}

		pBase += 4;
	}

#ifdef _DEBUG
	int nFreq = ((RAM[6004] & 0x01) << 0) | ((RAM[6005] & 0x01) << 1) |
		((RAM[6006] & 0x01) << 2) | ((RAM[6007] & 0x01) << 3);

	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left,     _rcDisplay.top,       RAM[GALAXIAN_SOUND_FREQUENCY] & 0x0F, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left + 8, _rcDisplay.top,      (RAM[GALAXIAN_SOUND_FREQUENCY] & 0xF0) >> 4, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left,     _rcDisplay.top + 8,   RAM[GALAXIAN_SOUND_SHOOT] & 0x0F, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left + 8, _rcDisplay.top + 8,  (RAM[GALAXIAN_SOUND_SHOOT] & 0xF0) >> 4, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left,     _rcDisplay.top + 16,  RAM[GALAXIAN_SOUND_VOLUME_F1] & 0x0F, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left + 8, _rcDisplay.top + 16, (RAM[GALAXIAN_SOUND_VOLUME_F1] & 0xF0) >> 4, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left,     _rcDisplay.top + 24,  RAM[GALAXIAN_SOUND_VOLUME_F2] & 0x0F, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left + 8, _rcDisplay.top + 24, (RAM[GALAXIAN_SOUND_VOLUME_F2] & 0xF0) >> 4, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left,     _rcDisplay.top + 32, (RAM[GALAXIAN_SOUND_NOISE] & 0x0F) >> 0, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left + 8, _rcDisplay.top + 32, (RAM[GALAXIAN_SOUND_NOISE] & 0xF0) >> 4, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left,     _rcDisplay.top + 40, (RAM[GALAXIAN_SOUND_SHOOT] & 0x0F) >> 0, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left + 8, _rcDisplay.top + 40, (RAM[GALAXIAN_SOUND_SHOOT] & 0xF0) >> 4, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left,     _rcDisplay.top + 48, (nFreq & 0x0F) >> 0, 1 << 2);
	_pimgChars->DrawChar(rgbBuffer, lPitch, _rcDisplay.left + 8, _rcDisplay.top + 48, (nFreq & 0xF0) >> 4, 1 << 2);
#endif
}

/*---------------------------------------------------------------------------

	Define game descriptors

---------------------------------------------------------------------------*/

GAME_DESCRIPTOR _gmdGalaxian = 
{
	TEXT("Galaxian\0|Galaxian"),
	(LPVOID)0x00000001,
	GalaxianInit,
	GalaxianExec,
	GalaxianRenderScene,
	GalaxianUpdateSound,
	GalaxianTerm
};

