/*---------------------------------------------------------------------------

	File:
		gorf.cpp
	
	Abstract:
		Implements the "Gorf" and "Wizard of Wor" game driver functions.

	Revisions:
		20-Feb-97 EAM Created

	Copyright (C) 1997 Edward Massey

	Adopted from Nicola Salmoria's MAME.  Currently in the process of
	translating his driver to MageX (not yet playable).

---------------------------------------------------------------------------*/

#include "pch.h"
#include <stdio.h>
#include "misc.h"
#include "../inc/z80api.h"
#include "game.h"
#include "game1.h"

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

enum 
{
    black, red, purple, cyan, yellow, blue, green, white
};

static const PALETTEENTRY _rgpe[] =
{
	{ 0x00, 0x00, 0x00 }, // black
	{ 0xFF, 0x00, 0x00 }, // red
	{ 0xB8, 0x02, 0xB8 }, // purple
	{ 0x00, 0xFF, 0xFF }, // cyan
	{ 0xFF, 0xFF, 0x00 }, // yellow
	{ 0x24, 0x24, 0xFF }, // blue
	{ 0x00, 0x90, 0x00 }, // green
	{ 0xFF, 0xFF, 0xFF }, // white
};

static const BYTE _rgbColorTable[] =
{
    black, yellow, blue, red
};

static HRESULT Init(GAME_LOAD_INFO &gli)
{
	switch ((DWORD)gli.pvContext)
	{
		case 0x00000000:
		{
			static const TCHAR szPath[] = "gorf";

			RAGE_ROMTABLE_BEGIN(ROMs)
				RAGE_ROMTABLE_ENTRY("gorf-a.bin", 0x0000, 0x1000)
				RAGE_ROMTABLE_ENTRY("gorf-b.bin", 0x1000, 0x1000)
				RAGE_ROMTABLE_ENTRY("gorf-c.bin", 0x2000, 0x1000)
				RAGE_ROMTABLE_ENTRY("gorf-d.bin", 0x3000, 0x1000)
				RAGE_ROMTABLE_ENTRY("gorf-e.bin", 0x8000, 0x1000)
				RAGE_ROMTABLE_ENTRY("gorf-f.bin", 0x9000, 0x1000)
				RAGE_ROMTABLE_ENTRY("gorf-g.bin", 0xa000, 0x1000)
				RAGE_ROMTABLE_ENTRY("gorf-h.bin", 0xb000, 0x1000)
			RAGE_ROMTABLE_END

			if (readroms(RAM, ROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
				return E_FAIL;

			break;
		}

		case 0x00000001:
		{
			static const TCHAR szPath[] = "wow";

			RAGE_ROMTABLE_BEGIN(ROMs)
				RAGE_ROMTABLE_ENTRY("wow.x1", 0x0000, 0x1000)
				RAGE_ROMTABLE_ENTRY("wow.x2", 0x1000, 0x1000)
				RAGE_ROMTABLE_ENTRY("wow.x3", 0x2000, 0x1000)
				RAGE_ROMTABLE_ENTRY("wow.x4", 0x3000, 0x1000)
				RAGE_ROMTABLE_ENTRY("wow.x5", 0x8000, 0x1000)
				RAGE_ROMTABLE_ENTRY("wow.x6", 0x9000, 0x1000)
				RAGE_ROMTABLE_ENTRY("wow.x7", 0xa000, 0x1000)
			RAGE_ROMTABLE_END

			if (readroms(RAM, ROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
				return E_FAIL;

			break;
		}

		default:
			return E_UNEXPECTED;
	}

	static const SIZE _sizBuffer = { 320, 205 };
	static const RECT _rcDisplay = { 0, 0, 320, 204 };

	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	gli.cpe = 256;
	
	for (int i = 0; i < gli.cpe; i++)
	{
		gli.ppe[i].peRed   = MulDiv(i, 256, gli.cpe);
		gli.ppe[i].peGreen = MulDiv(i, 256, gli.cpe);
		gli.ppe[i].peBlue  = MulDiv(i, 256, gli.cpe);
	}
	
//	BuildCompositePalette(_rgpe, DIM(_rgpe), _rgbColorTable,
//		COLOR_CODES * 4, gli.ppe, 256, gli.cpe);

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static BYTE M_RDMEM(WORD A)
{
	return RAM[A];
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static void M_WRMEM(WORD A, BYTE V)
{
	if (A >= 0x0000 && A <= 0x3fff)
	{
		// masked video RAM
	}
	else if (A >= 0xd000 && A <= 0xd3ff)
	{
		RAM[A] = V;
	}
	else if (A >= 0x4000 && A <= 0x7fff)
	{
		RAM[A] = V; // video RAM
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static int interrupt_vector = 0xff;

static int Interrupt(void)
{
	if (!CheckMessages())
		CPURunning = 0;

	Throttle(60); // FRAMES_PER_SECOND);

	if (RAM[0xfff0] == 0)
		return IGNORE_INT;
	
	return interrupt_vector;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static int collision, mask, unknown;

static BYTE In(BYTE bAddr)
{
	switch (bAddr)
	{
		case 0x08:
		{
			return collision;
		}

		case 0x10:
		{
			BYTE b = 0xFF;

			if (osd_key_pressed(OSD_KEY_3))
				b ^= 0x01;

			if (osd_key_pressed(OSD_KEY_F2))
				b ^= 0x08;

			if (osd_key_pressed(OSD_KEY_1))
				b ^= 0x20;

			if (osd_key_pressed(OSD_KEY_2))
				b ^= 0x40;

			if (osd_key_pressed(OSD_KEY_B))
				b ^= 0x80;

			return b;
		}

		case 0x11:
		{
			BYTE b = 0xEF;

			if (osd_key_pressed(OSD_KEY_E))
				b ^= 0x01;

			if (osd_key_pressed(OSD_KEY_D))
				b ^= 0x02;

			if (osd_key_pressed(OSD_KEY_S))
				b ^= 0x04;

			if (osd_key_pressed(OSD_KEY_F))
				b ^= 0x08;

			if (osd_key_pressed(OSD_KEY_X))
				b ^= 0x10;

			if (osd_key_pressed(OSD_KEY_X))
				b ^= 0x20;

			return b;
		}

		case 0x12:
		{
			BYTE b = 0xEF;

			if (osd_key_pressed(OSD_KEY_UP))
				b ^= 0x01;

			if (osd_key_pressed(OSD_KEY_DOWN))
				b ^= 0x02;

			if (osd_key_pressed(OSD_KEY_LEFT))
				b ^= 0x04;

			if (osd_key_pressed(OSD_KEY_RIGHT))
				b ^= 0x08;

			if (osd_key_pressed(OSD_KEY_ALT))
				b ^= 0x10;

			if (osd_key_pressed(OSD_KEY_CONTROL))
				b ^= 0x20;

			return b;
		}

		case 0x13:
		{
			BYTE b = 0xFF;

			return b;
		}

		default:
			return 0x00;
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

// NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  
//
// credits for the blitting code here go to Nicola Salmoria.  I have not 
// modified this code very much
//
// NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  NOTE  

static void Out(BYTE bAddr, BYTE bVal)
{
	switch (bAddr)
	{
		case 0x0D:
		{
			interrupt_vector = bVal;
			break;
		}

		case 0x19: // wow_mask_w
		{
			mask = bVal;
			break;
		}

		case 0x0C: // wow_unknown_w
		{
			unknown = bVal;
			break;
		}

		case 0x78: // wow_blitter_w
		case 0x79: // wow_blitter_w
		case 0x7A: // wow_blitter_w
		case 0x7B: // wow_blitter_w
		case 0x7C: // wow_blitter_w
		case 0x7D: // wow_blitter_w
		case 0x7E: // wow_blitter_w
		{
			static int src;
			static int mode;	/* ?? */
			static int skip;	/* bytes to skip after row copy */
			static int dest;
			static int length;	/* row length */
			static int loops;	/* rows to copy - 1 */


			switch (bAddr - 0x78)
			{
				case 0:
					src = bVal;
					break;
				case 1:
					src = src + bVal * 256;
					break;
				case 2:
					mode = bVal & 0x3f;	/* register is 6 bit wide */
					break;
				case 3:
					skip = bVal;
					break;
				case 4:
					dest = skip + bVal * 256;	/* register 3 is shared between skip and dest */
					break;
				case 5:
					length = bVal;
					break;
				case 6:
					loops = bVal;
					break;
			}

			if (bAddr - 0x78 == 6)	/* trigger blit */
			{
				int i,j;

				for (i = 0; i <= loops;i++)
				{
					for (j = 0;j <= length;j++)
					{
						if (!(mode & 0x08) || j < length)
							RAM[dest] = RAM[src]; // Z80_WRMEM(dest,RAM[src]);

						if (mode & 0x20)
							dest++;	/* copy forwards */
						else
							dest--;				/* backwards */

						if ((j & 1) || !(mode & 0x02))	/* don't increment source on odd loops */
							if (mode & 0x04) src++;
					}

					if ((j & 1) && (mode & 0x02))	/* always increment source at end of line */
				
					if (mode & 0x04) src++;
					if (mode & 0x08) src--;

					if (mode & 0x20) dest--;	/* copy forwards */
					else dest++;				/* backwards */

					dest += (int)((signed char)skip);	/* extend the sign of the skip register */
				}
			}

			break;
		}
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

// TODO:  call higher-level Exec function that handles the common parts
//        of Interrupt, pass it our CPU speed and frame rate

static HRESULT Run(LPVOID pvContext)
{
	EXEC_Z80_STRUCT es;
	
	ZERO_INIT(es);
	
	es.nCallbackFreq = 50000;
	es.pfnInterrupt = Interrupt;
	es.pfnReadMemory = M_RDMEM;
	es.pfnWriteMemory = M_WRMEM;
	es.rgbMemory = RAM;
	es.pfnIn = In;
	es.pfnOut = Out;
	
	return ExecZ80(es);
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static HRESULT RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
/*
	LPBYTE pBase = &RAM[0x0000];

	for (int i = 0; i < 204; i++)
	{
		for (int j = 0; j < 256; j++)
		{
			rgbBuffer[j + i * lPitch] = *pBase++;
		}
	}
*/

	int i;
	
	LPBYTE pBase = &RAM[0x4000];

	for (i = 0; i < 0x4000; i++)
	{
		BYTE b = *pBase++;

		int sx = 8 * (i % 40);
		int sy = (i / 2) / 40;

		rgbBuffer[sy * lPitch + sx + 0] = (b & 0xC0) >> 6;
		rgbBuffer[sy * lPitch + sx + 1] = (b & 0x30) >> 4;
		rgbBuffer[sy * lPitch + sx + 2] = (b & 0x0C) >> 2;
		rgbBuffer[sy * lPitch + sx + 3] = (b & 0x03) >> 0;
	}

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

GAME_DESCRIPTOR _gmdGorf = 
{
	TEXT("Gorf"),
	Init,
	Run,
	RenderScene,
	NULL,
	NULL,
	NULL,
	NULL
};

GAME_DESCRIPTOR _gmdWizardOfWor = 
{
	TEXT("Wizard of Wor"),
	Init,
	Run,
	RenderScene,
	NULL,
	NULL,
	NULL,
	(LPVOID)0x00000001
};

