/*---------------------------------------------------------------------------

	File:
		blit.h
	
	Abstract:
		Sprite and character blitting routines.

	Revisions:
		10-Feb-97 EAM Created

	Copyright (C) 1997 Edward Massey

	CONSIDER:  align blits on 32-bit boundary
	CONSIDER:  interleave mask and color data to eliminate pMask
	CONSIDER:  pass rgbSource to remove dependency on globals

---------------------------------------------------------------------------*/

#ifndef _BLIT_H_
#define _BLIT_H_

typedef const void *LPCVOID;
typedef const BYTE *LPCBYTE;
typedef const DWORD far *LPCDWORD;

#ifdef _DEBUG

void DrawChar(LPCBYTE rgbSource, int offset_value,
	int color, int X, int Y, LPBYTE rgbBuffer, LONG lPitch);

void DrawChar(LPCBYTE rgbSource, int offset_value,
	int X, int Y, LPBYTE rgbBuffer, LONG lPitch);

void DrawSprite(LPCBYTE rgbData, LPCBYTE rgbMask,
	int n, int color, int x, int y, LPBYTE rgbBuffer, LONG lPitch);

void DrawSprite2(LPBYTE rgbBuffer, LONG lPitch, int x, int y,
	int n, int color, LPCBYTE rgbData, LPCBYTE rgbMask);

void DrawCharT(LPCBYTE rgbData, LPCBYTE rgbMask,
	int n, int color, int x, int y, LPBYTE rgbBuffer, LONG lPitch);

#else

inline static void DrawChar(LPCBYTE rgbSource, int offset_value,
	int color, int X, int Y, LPBYTE rgbBuffer, LONG lPitch)
{
	LPDWORD pDest = (LPDWORD)&rgbBuffer[lPitch * Y + X];
	LPCDWORD pSrc = (LPCDWORD)&rgbSource[offset_value * 8 * 8];

	// distribute color selector into all four bytes

	color |= (color << 8) | (color << 16) | (color << 24);

	lPitch = (lPitch / 4) - 1;

	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc++); pDest += lPitch;
	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc++); pDest += lPitch;
	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc++); pDest += lPitch;
	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc++); pDest += lPitch;
	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc++); pDest += lPitch;
	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc++); pDest += lPitch;
	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc++); pDest += lPitch;
	*pDest++ = (color | *pSrc++); *pDest = (color | *pSrc);
}

/*

Draw an opaque 8x8 image using only the color information contained in the
image data itself.

This routine is used by Donkey Kong, Donkey Kong Jr., and Mario Bros., since
the character backgrounds color is defined by the character itself, instead
of a color VRAM bank, as is typical in the other games.

*/

inline static void DrawChar(LPCBYTE rgbSource, int offset_value,
	int X, int Y, LPBYTE rgbBuffer, LONG lPitch)
{
	LPDWORD pDest = (LPDWORD)&rgbBuffer[lPitch * Y + X];
	LPCDWORD pSrc = (LPCDWORD)&rgbSource[offset_value * 8 * 8];

	lPitch = (lPitch / 4) - 1;

	*pDest++ = *pSrc++; *pDest = *pSrc++; pDest += lPitch;
	*pDest++ = *pSrc++; *pDest = *pSrc++; pDest += lPitch;
	*pDest++ = *pSrc++; *pDest = *pSrc++; pDest += lPitch;
	*pDest++ = *pSrc++; *pDest = *pSrc++; pDest += lPitch;
	*pDest++ = *pSrc++; *pDest = *pSrc++; pDest += lPitch;
	*pDest++ = *pSrc++; *pDest = *pSrc++; pDest += lPitch;
	*pDest++ = *pSrc++; *pDest = *pSrc++; pDest += lPitch;
	*pDest++ = *pSrc++; *pDest = *pSrc;
}

// consider: use a function template, parameterized with number of color
// schemes, bit depth of sprites, put in header file (inline)

inline static void DrawSprite(LPCBYTE rgbData, LPCBYTE rgbMask,
	int n, int color, int x, int y, LPBYTE rgbBuffer, LONG lPitch)
{
	//
	//
	//
	
	color |= (color << 8) | (color << 16) | (color << 24);

	//
	//
	//
	
	LPDWORD pDest = (LPDWORD)&rgbBuffer[lPitch * y + x];
	LPCDWORD pSrc = (LPCDWORD)&rgbData[n * 16 * 16];
	LPCDWORD pMask = (LPCDWORD)&rgbMask[n * 16 * 16];
	
	//
	//  Draw sprite, with transparency, and color scheme adjusted
	//  to offscreen buffer, 32-bits at a time
	//
	//  CONSIDER:  unroll the loop
	//
	
	lPitch = (lPitch / 4) - 4;
	
#if 0
	for (int i = 0; i < 16; i++)
	{
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;
	}

#else
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
#endif
}

inline static void DrawSprite2(LPBYTE rgbBuffer, LONG lPitch, int x, int y,
	int n, int color, LPCBYTE rgbData, LPCBYTE rgbMask)
{
	//
	//
	//
	
	color |= (color << 8) | (color << 16) | (color << 24);

	//
	//
	//
	
	LPDWORD pDest = (LPDWORD)&rgbBuffer[lPitch * y + x];
	LPCDWORD pSrc = (LPCDWORD)&rgbData[n * 32 * 32];
	LPCDWORD pMask = (LPCDWORD)&rgbMask[n * 32 * 32];
	
	//
	//  Draw sprite, with transparency, and color scheme adjusted
	//  to offscreen buffer, 32-bits at a time
	//
	//  CONSIDER:  unroll the loop
	//
	
	lPitch = (lPitch / 4) - 8;
	
	for (int i = 0; i < 8; i++)
	{
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		pDest += lPitch;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;

		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
		pDest++; pMask++; pSrc++;
		*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	}
}

inline static void DrawCharT(LPCBYTE rgbData, LPCBYTE rgbMask,
	int n, int color, int x, int y, LPBYTE rgbBuffer, LONG lPitch)
{
	LPDWORD pDest = (LPDWORD)&rgbBuffer[lPitch * y + x];
	LPCDWORD pSrc = (LPCDWORD)&rgbData[n * 8 * 8];
	LPCDWORD pMask = (LPCDWORD)&rgbMask[n * 8 * 8];

	color |= (color << 8) | (color << 16) | (color << 24);

	lPitch = (lPitch / 4) - 1;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;

	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest++; pMask++; pSrc++;
	*pDest = (*pDest & *pMask) | ((color | *pSrc) & ~(*pMask));
	pDest += lPitch; pMask++; pSrc++;
}

#endif

#endif // _BLIT_H_

