/*---------------------------------------------------------------------------

	File:
		ex6502i.h

	Abstract:
		Internal declarations for Motorola 6502 emulator.

	Revisions:
		07-Apr-97 EAM Created

	Copyright (C) 1997 Edward Massey

	This library is free software; you can redistribute it and/or modify it
	under the terms of version 2 of the GNU Library General Public License
	as published by the Free Software Foundation.

	This library is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
	General Public License for more details.
	To obtain a copy of the GNU Library General Public License, write to the
	Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Any permitted reproduction of these routines, in whole or in part, must
	bear this legend.

---------------------------------------------------------------------------*/

#ifndef _EX6502I_H_
#define _EX6502I_H_

#define EXEC6502_API extern "C" __declspec(dllexport)

#include "exec6502.h"

class CExec6502 : public IExec6502
{
public:
	CExec6502(EXEC6502_CREATE &cs);

	virtual ~CExec6502();

	// *** IUnknown methods ***
	virtual HRESULT __stdcall QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject);
	virtual ULONG __stdcall AddRef();
	virtual ULONG __stdcall Release();

	// *** IExec6502 methods ***
	virtual void SetContext(const EXEC6502_CONTEXT &ctx);
	virtual void GetContext(EXEC6502_CONTEXT &ctx);
	virtual void Reset();
	virtual int Exec(int cCycles);
	virtual int Nmi();
	virtual int Irq();

protected:
	inline void PushByte(BYTE bVal);
	inline BYTE PullByte();

	inline void PushWord(WORD wVal);
	inline WORD PullWord();

	inline BYTE ImmedByte();
	inline WORD ImmedWord();

	inline WORD ReadWord(WORD wAddr);
	inline BYTE ReadByte(WORD wAddr);

	BYTE CallbackReadByte(WORD wAddr);

	inline void MemWrite(WORD wAddr, BYTE bVal);

	void CallbackMemWrite(WORD wAddr, BYTE bVal);

	inline void Flags(BYTE b);

	static int ZNTable[];
	static int NoOvTable[];

	// operations...
	inline void Adc(BYTE bArg);
	inline void Sbc(BYTE bArg);
	inline void Cmp(BYTE bArg1, BYTE bArg2);
	inline void Bit(BYTE bArg);
	inline void And(BYTE bArg);
	inline void Ora(BYTE bArg);
	inline void Eor(BYTE bArg);
	inline BYTE Asl(BYTE bArg);
	inline BYTE Lsr(BYTE bArg);
	inline BYTE Rol(BYTE bArg);
	inline BYTE Ror(BYTE bArg);
	inline BYTE Inc(BYTE bArg);
	inline BYTE Dec(BYTE bArg);

	// address mode calculations...
	inline BYTE MR_Zx();
	inline BYTE MR_Zy();
	inline BYTE MR_Ax();
	inline BYTE MR_Ay();
	inline BYTE MR_Ix();
	inline BYTE MR_Iy();
	inline BYTE MR_Zp();
	inline BYTE MR_Ab();
	inline void MW_Zx(BYTE bVal);

	inline int Jr0(BOOL f);
	inline int Jr1(BOOL f);

	void Break();

	inline void SetPC(WORD wAddr);
	inline WORD GetPC();

private:
	ULONG m_cRef;

	BYTE *const m_rgbMemory; // direct access to memory buffer
	const BYTE *m_rgbOpcode; // &m_rgbMemory[m_regPC]
	BYTE *m_rgbStack;        // &m_rgbMemory[m_regS];

	BYTE m_regP;  // processor status
	BYTE m_regA;  // accumulator
	BYTE m_regX;  // x index
	BYTE m_regY;  // y index

	BYTE m_rgMemDesc[0x10000];
	EXEC6502_MEMREAD_PROC m_rgpfnRead[0x10000];
	EXEC6502_MEMWRITE_PROC m_rgpfnWrite[0x10000];
};

#define M6502_NMI_VECTOR 0xFFFA
#define M6502_RST_VECTOR 0xFFFC
#define M6502_IRQ_VECTOR 0xFFFE

#endif // _EX6502I_H_

