/*---------------------------------------------------------------------------

	File:
		exec6502.h

	Abstract:
		Declarations for Motorola 6502 emulator.

	Revisions:
		07-Apr-97 EAM Created

	Copyright (C) 1997 Edward Massey

	This library is free software; you can redistribute it and/or modify it
	under the terms of version 2 of the GNU Library General Public License
	as published by the Free Software Foundation.

	This library is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
	General Public License for more details.
	To obtain a copy of the GNU Library General Public License, write to the
	Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Any permitted reproduction of these routines, in whole or in part, must
	bear this legend.

---------------------------------------------------------------------------*/

#ifndef _EXEC6502_H_
#define _EXEC6502_H_

//
//  processor status (regP) flags
//

#define C_FLAG   0x01
#define Z_FLAG   0x02
#define I_FLAG   0x04
#define D_FLAG   0x08
#define B_FLAG   0x10
#define R_FLAG   0x20
#define V_FLAG   0x40
#define N_FLAG   0x80

struct EXEC6502_CONTEXT
{
	WORD regPC;
	BYTE regA, regX, regY, regS, regP;
};

#define MF_READ_DIRECT    0x00000001
#define MF_WRITE_DIRECT   0x00000002
#define MF_READ_ILLEGAL   0x00000004
#define MF_WRITE_ILLEGAL  0x00000008
#define MF_ADJUST_ADDRESS 0x00000010
#define MF_RESERVED       0xFFFFFFE0

typedef BYTE (*EXEC6502_MEMREAD_PROC)(WORD);
typedef void (*EXEC6502_MEMWRITE_PROC)(WORD, BYTE);

struct EXEC6502_MEM_DESCRIPTOR
{
	DWORD dwAddrMin;
	DWORD dwAddrMax;
	DWORD dwFlags;
	EXEC6502_MEMREAD_PROC pfnRead;
	EXEC6502_MEMWRITE_PROC pfnWrite;
};

#define BEGIN_MEMORY_MAP(nm) static const EXEC6502_MEM_DESCRIPTOR (nm)[] = {
#define MEMORY_MAP_ENTRY { 0, 0, 0, 0, 0 }
#define END_MEMORY_MAP { 0, 0, 0, (EXEC6502_READ_PROC)0xFFFFFFFF, 0 } };

struct EXEC6502_CREATE
{
	DWORD dwFlags;
    CLSID clsid;
	LPBYTE rgbMemory;
	const EXEC6502_MEM_DESCRIPTOR *pmd;
};

struct IExec6502 : IUnknown
{
	virtual void GetContext(EXEC6502_CONTEXT &ctx) = 0;
	virtual void SetContext(const EXEC6502_CONTEXT &ctx) = 0;
	virtual void Reset() = 0;
	virtual int  Exec(int cCycles) = 0;
	virtual int  Nmi() = 0;
	virtual int  Irq() = 0;
};

#ifndef EXEC6502_API
#define EXEC6502_API extern "C" __declspec(dllimport)
#endif

EXEC6502_API HRESULT EclCreate6502(EXEC6502_CREATE &cs, IExec6502 *&pExec);
EXEC6502_API int EclDasm6502(LPTSTR S, const BYTE *A, unsigned long PC);

#endif // _EXEC6502_H_

