/*---------------------------------------------------------------------------

	File:
		ay8910.h
	
	Abstract:
		Emulates the AY-3-8910 (and compatible) family of sound chips.

	Revisions:
		02-Jun-97 EAM Created

	Copyright (C) 1997 Edward Massey

	References:  sound.c, by Ville Hallik, for MSX emulator; Mike Cuddy for
		psg.c module in MAME

---------------------------------------------------------------------------*/

#ifndef _AY8910_H_
#define _AY8910_H_

#define AY_AFINE	(0)
#define AY_ACOARSE	(1)
#define AY_BFINE	(2)
#define AY_BCOARSE	(3)
#define AY_CFINE	(4)
#define AY_CCOARSE	(5)
#define AY_NOISEPER	(6)
#define AY_ENABLE	(7)
#define AY_AVOL		(8)
#define AY_BVOL		(9)
#define AY_CVOL		(10)
#define AY_EFINE	(11)
#define AY_ECOARSE	(12)
#define AY_ESHAPE	(13)

#define AY_PORTA	(14)
#define AY_PORTB	(15)



/* SAMPLE_16BIT is not supported, yet */
#ifndef SAMPLE_16BIT
typedef unsigned char SAMPLE;
#define SIGNED_SAMPLES
#ifdef SIGNED_SAMPLES
#define AUDIO_CONV(A) (128+(A))	/* use this macro for signed samples */
#else
#define AUDIO_CONV(A) (A)		/* use this macro for unsigned samples */
#endif
#else
typedef unsigned short SAMPLE;
#endif

struct AY8910_BASE
{
	BYTE m_rgbRegs[16];

    int m_Incr0, m_Incr1, m_Incr2;
    int m_Increnv, m_Incrnoise;
    int m_StateNoise, m_NoiseGen;
    int m_Counter0, m_Counter1, m_Counter2, m_Countenv, m_Countnoise;
    int m_Vol0, m_Vol1, m_Vol2, m_Volnoise, m_Envelope;

	AY8910_BASE();
	void Reset();
	void WriteReg(int r, int v);
	BYTE ReadReg(int r);
	void UpdateSound(LPBYTE rgbBuffer, LONG cbBuffer);
	void UpdateSound(LPINT rgbBuffer, LONG cbBuffer);

	static const BYTE _rgbEnvelopes[16][32];

	int m_latch;

	int (*m_pfnReadPortA)();
	int (*m_pfnReadPortB)();
	void (*m_pfnWritePortA)(int);
	void (*m_pfnWritePortB)(int);
};

struct AY8910 : AY8910_BASE
{
	void WriteRegLatch(int v)
	{
		WriteReg(m_latch, v);
	}

	BYTE ReadRegLatch()
	{
		return ReadReg(m_latch);
	}

	void WriteLatch(int n)
	{
		m_latch = n;
	}

	BYTE ReadLatch()
	{
		return m_latch;
	}

	int m_latch;
};

#endif // _AY8910_H_

