/*---------------------------------------------------------------------------

	File:
		game1.cpp
	
	Abstract:
		Common routines for game drivers.

	Revisions:
		05-Feb-97 EAM Created

	Copyright (C) 1997 Edward Massey

	We'll put a generic loader here in the future.  It'll do the following:
	- load code ROMs
	- load, decode, rotate, and build masks for graphics ROMs
	- build composite palette

---------------------------------------------------------------------------*/

#include "pch.h"
#include <time.h>
#include "../inc/game1.h"
#include "misc.h"

int readroms(unsigned char *dest, const struct RomModule * romp,
	const char *basename, LPTSTR pszMsg, UINT cchMsgMax)
{
	FILE		   *f;
	char			buf[100];
	char			name[100];

	while (romp->name)
	{
		sprintf(buf, romp->name, basename);
		sprintf(name, "%s/%s", basename, buf);

		if ((f = fopen(name, "rb")) == 0)
		{
			sprintf(pszMsg, "Unable to open file %s\n", name);
			return 1;
		}
		if (fread(dest + romp->offset, 1, romp->size, f) != romp->size)
		{
			sprintf(pszMsg, "Unable to read file %s\n", name);
			fclose(f);
			return 1;
		}
		fclose(f);

		romp++;
	}

	return 0;
}

int readroms(unsigned char *dest, const struct RomModule2 * romp,
	const char *basename, LPTSTR pszMsg, UINT cchMsgMax)
{
	FILE		   *f;
	char			buf[100];
	char			name[100];

	while (romp->name)
	{
		sprintf(buf, romp->name, basename);
		sprintf(name, "%s/%s", basename, buf);

		if ((f = fopen(name, "rb")) == 0)
		{
			sprintf(pszMsg, "Unable to open file %s\n", name);
			return 1;
		}
		if (fseek(f, romp->offset, SEEK_SET) != 0)
		{
			sprintf(pszMsg, "Unable to read file %s\n", name);
			fclose(f);
			return 1;
		}
		if (fread(dest + romp->addr, 1, romp->size, f) != romp->size)
		{
			sprintf(pszMsg, "Unable to read file %s\n", name);
			fclose(f);
			return 1;
		}
		fclose(f);

		romp++;
	}

	return 0;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

FILE *_pfTrace = NULL;

extern "C" void TRACE(LPCTSTR pszFormat, ...)
{
#if 1 // def _DEBUG
	TCHAR szBuf[512];
	
	va_list marker;
	va_start(marker, pszFormat);
	
	_vsnprintf(szBuf, DIM(szBuf), pszFormat, marker);
	
	OutputDebugString(szBuf);

	if (_pfTrace == NULL)
		_pfTrace = fopen("debug.txt", "w");

	if (_pfTrace != NULL)
	{
		fputs(szBuf, _pfTrace);
		fflush(_pfTrace);
	}
#endif
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

BOOL ReadControl(const WORD A, const CONTROL *pCtrl, BYTE &b)
{
	for (; pCtrl->A != 0xFFFF; pCtrl++)
	{
		if (pCtrl->A != A)
			continue;

		//
		//
		//
		
		b = pCtrl->bDefault;

		const int *pn = pCtrl->rgnKey;

		for (int j = 0; j < 8; j++)
			b ^= ((osd_key_pressed(*pn++)) << j);

		return TRUE;
	}

	return FALSE;
}
