/*---------------------------------------------------------------------------

	File:
		execz80.h

	Abstract:
		Declarations for Zilog Z80 emulator interface as part of the
		Emulation Component Library for MageX.

	Revisions:
		08-Apr-97 EAM Created

	Copyright (C) 1997 Edward Massey

	This library is free software; you can redistribute it and/or modify it
	under the terms of version 2 of the GNU Library General Public License
	as published by the Free Software Foundation.

	This library is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
	General Public License for more details.
	To obtain a copy of the GNU Library General Public License, write to the
	Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Any permitted reproduction of these routines, in whole or in part, must
	bear this legend.

---------------------------------------------------------------------------*/

#ifndef _EXECZ80_H_
#define _EXECZ80_H_

//
//
//

#define S_FLAG          0x80
#define Z_FLAG          0x40
#define H_FLAG          0x10
#define V_FLAG          0x04
#define N_FLAG          0x02
#define C_FLAG          0x01

//
//
//

struct EXECZ80_CONTEXT
{
	WORD AF,BC,DE,HL,IX,IY,PC,SP;
	WORD AF2,BC2,DE2,HL2;
	unsigned IFF1,IFF2,HALT,IM,I,R,R2;
	int IPeriod, ICount;
};

typedef BYTE (*EXECZ80_MEMREAD_PROC)(WORD);
typedef void (*EXECZ80_MEMWRITE_PROC)(WORD, BYTE);
typedef BYTE (*EXECZ80_PORTREAD_PROC)(BYTE);
typedef void (*EXECZ80_PORTWRITE_PROC)(BYTE, BYTE);

#define Z80_END_TABLE     ((EXECZ80_MEMREAD_PROC) 0xFFFFFFFF)
#define Z80_IGNORE_READ   ((EXECZ80_MEMREAD_PROC) 0xFFFFFFFE)
#define Z80_IGNORE_WRITE  ((EXECZ80_MEMWRITE_PROC)0xFFFFFFFE)
#define Z80_ILLEGAL_READ  ((EXECZ80_MEMREAD_PROC) 0xFFFFFFFD)
#define Z80_ILLEGAL_WRITE ((EXECZ80_MEMWRITE_PROC)0xFFFFFFFD)

struct EXECZ80_MEM_DESCRIPTOR
{
	DWORD dwAddrMin;
	DWORD dwAddrMax;
	DWORD dwFlags;
	EXECZ80_MEMREAD_PROC pfnMemRead;
	EXECZ80_MEMWRITE_PROC pfnMemWrite;
};

struct EXECZ80_CREATE
{
	DWORD dwFlags;
	LPBYTE rgbMemory;
	const BYTE *rgbInstructions;
	EXECZ80_MEMREAD_PROC pfnMemRead;
	EXECZ80_MEMWRITE_PROC pfnMemWrite;
	EXECZ80_PORTREAD_PROC pfnPortRead;
	EXECZ80_PORTWRITE_PROC pfnPortWrite;
	const EXECZ80_MEM_DESCRIPTOR *pmd;
};

interface IExecZ80 : public IUnknown
{
	virtual void GetContext(EXECZ80_CONTEXT &ctx) = 0;
	virtual void SetContext(const EXECZ80_CONTEXT &ctx) = 0;
	virtual void Reset() = 0;
	virtual int Exec(int cTStates) = 0;
	virtual int Nmi() = 0;
	virtual int Irq(BYTE bVal) = 0;
};

#ifndef EXECZ80_API
#define EXECZ80_API extern "C" __declspec(dllimport)
#endif

EXECZ80_API HRESULT EclCreateZ80(EXECZ80_CREATE &cs, IExecZ80 *&pExec);

#endif // _EXECZ80_H_

