/*---------------------------------------------------------------------------

	File:
		tables.cpp

	Abstract:
		Flag tables for Z80 emulation

	Revisions:
		18-Apr-97 EAM Created

	Copyright (C) 1997 Edward Massey

	NOTE:  In the current version, the z80daa.h file and table
		initialization routine are derived from the Portable Z80 Emulator by
		Marat Fayzullin and Marcel de Kogel.

---------------------------------------------------------------------------*/

#include "pch.h"
#include "execz80i.h"

BYTE PTable[512];
BYTE ZSTable[512];
BYTE ZSPTable[512];
BYTE rgfAdd[256][256];
BYTE rgfSub[256][256];
BYTE rgfAddc[256][256][2];
BYTE rgfSubc[256][256][2];

#if 1
#include "z80daa.h"
#else
short DAATable[2048];
#endif

void InitTables (void)
{
	byte zs;

	int i,p;

	//
	//
	//
	
	for (i=0; i<256; ++i)
	{
		zs=0;
		
		if (i==0)
			zs|=Z_FLAG;
		
		if (i&0x80)
			zs|=S_FLAG;
		
		p=0;
		
		if (i&1) ++p;
		if (i&2) ++p;
		if (i&4) ++p;
		if (i&8) ++p;
		if (i&16) ++p;
		if (i&32) ++p;
		if (i&64) ++p;
		if (i&128) ++p;
		
		PTable[i]  = (p&1) ? 0:V_FLAG;
		ZSTable[i] = zs;
		ZSPTable[i]= zs | PTable[i];
	}

	//
	//
	//
	
	for (i=0;i<256;++i)
	{
		ZSTable[i + 256]  = ZSTable[i]  | C_FLAG;
		ZSPTable[i + 256] = ZSPTable[i] | C_FLAG;
		PTable[i + 256]   = PTable[i]   | C_FLAG;
	}

	//
	//  Calculate flag tables
	//
	
	for (DWORD dw = 0; dw <= 256 * 256; dw++)
	{
		BYTE bLo = LOBYTE(LOWORD(dw));
		BYTE bHi = HIBYTE(LOWORD(dw));
		
		const BYTE regA = bLo;
		const BYTE bArg = bHi;

		int nAcc;
		BYTE regF;

		//
		//  ADD
		//
		
		nAcc = regA + bArg;
		regF = 0;
		
		regF |= ZSTable[nAcc&255];
		regF |= ((nAcc&256)>>8);
		regF |= ((regA^nAcc^bArg)&H_FLAG);
		regF |= (((bArg^regA^0x80)&(bArg^nAcc)&0x80)>>5);

		rgfAdd[regA][bArg] = regF;

		//
		//  SUB
		//
		
		nAcc  = regA - bArg;
		regF = ZSTable[nAcc&255]|((nAcc&256)>>8)|N_FLAG|((regA^nAcc^bArg)&H_FLAG)|(((bArg^regA)&(bArg^nAcc)&0x80)>>5);

		rgfSub[regA][bArg] = regF;

		//
		//
		//

		for (int c = 0; c <= 1; c++)
		{
			//
			//  ADC
			//

			nAcc = regA + bArg + c;
			regF = 0;
			
			regF |= ZSTable[nAcc&255];
			regF |= ((nAcc&256)>>8);
			regF |= ((regA^nAcc^bArg)&H_FLAG);
			regF |= (((bArg^regA^0x80)&(bArg^nAcc)&0x80)>>5);

			rgfAddc[regA][bArg][c] = regF;

			//
			//  SBC
			//

			nAcc = regA - bArg - c;
			regF = ZSTable[nAcc&255]|((nAcc&256)>>8)|N_FLAG|((regA^nAcc^bArg)&H_FLAG)|(((bArg^regA)&(bArg^nAcc)&0x80)>>5);

			rgfSubc[regA][bArg][c] = regF;
		}
	}
}

